/******************************************************************************
 * Copyright (c) 2006-2012 Quality & Usability Lab                            *
 *                         Deutsche Telekom Laboratories, TU Berlin           *
 *                         Ernst-Reuter-Platz 7, 10587 Berlin, Germany        *
 *                                                                            *
 * This file is part of the SoundScape Renderer (SSR).                        *
 *                                                                            *
 * The SSR is free software:  you can redistribute it and/or modify it  under *
 * the terms of the  GNU  General  Public  License  as published by the  Free *
 * Software Foundation, either version 3 of the License,  or (at your option) *
 * any later version.                                                         *
 *                                                                            *
 * The SSR is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY;  without even the implied warranty of MERCHANTABILITY or FITNESS *
 * FOR A PARTICULAR PURPOSE.                                                  *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * You should  have received a copy  of the GNU General Public License  along *
 * with this program.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * The SSR is a tool  for  real-time  spatial audio reproduction  providing a *
 * variety of rendering algorithms.                                           *
 *                                                                            *
 * http://tu-berlin.de/?id=ssr                  SoundScapeRenderer@telekom.de *
 ******************************************************************************/

/** @file
 * Connection class (definition).
 *
 * $LastChangedDate: 2012-02-01 11:43:25 +0100 (Wed, 01 Feb 2012) $
 * $LastChangedRevision: 1659 $
 * $LastChangedBy: geier.matthias $
 **/

#ifndef CONNECTION_H
#define CONNECTION_H

#ifdef HAVE_CONFIG_H
#include <config.h> // for ENABLE_*
#endif

#include <boost/asio.hpp>
#ifdef ENABLE_SSL
#include <boost/asio/ssl.hpp>
#endif
#include <boost/enable_shared_from_this.hpp>
#include <iostream>

#include "networksubscriber.h"
#include "commandparser.h"

class Controller;

/// Connection class.
class Connection : public boost::enable_shared_from_this<Connection>
{
  public:
    /// Ptr to Connection
    typedef boost::shared_ptr<Connection> pointer;
#ifdef ENABLE_SSL
    typedef boost::asio::ssl::stream<boost::asio::ip::tcp::socket> socket_t;
#else
    typedef boost::asio::ip::tcp::socket socket_t;
#endif

    static pointer create(boost::asio::io_service &io_service
#ifdef ENABLE_SSL
        , boost::asio::ssl::context& context
#endif
        , Controller &controller);

    void start();
    void write(std::string &writestring);

    /// @return Reference to socket
#ifdef ENABLE_SSL
    socket_t::lowest_layer_type& socket() { return _socket.lowest_layer(); }
#else
    socket_t& socket() { return _socket; }
#endif

    ~Connection();

  private:
    Connection(boost::asio::io_service &io_service
#ifdef ENABLE_SSL
        , boost::asio::ssl::context& context
#endif
        , Controller &controller);

    void start_read();
    void read_handler(const boost::system::error_code &error, size_t size);
    void write_handler(boost::shared_ptr<std::string> str_ptr
        , const boost::system::error_code &error, size_t bytes_transferred);

    void timeout_handler(const boost::system::error_code &e);

#ifdef ENABLE_SSL
    void handle_handshake(const boost::system::error_code& error);
#endif

    /// TCP/IP socket
    socket_t _socket;
    /// Buffer for incoming messages.  
    boost::asio::streambuf _streambuf;
    /// @see Connection::timeout_handler
    boost::asio::deadline_timer _timer;

    /// Reference to Controller
    Controller &_controller;
    /// Subscriber obj
    NetworkSubscriber _subscriber;
    /// Commandparser obj 
    CommandParser _commandparser;
};

#endif

// Settings for Vim (http://www.vim.org/), please do not remove:
// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
