/******************************************************************************
 * Copyright (c) 2006-2012 Quality & Usability Lab                            *
 *                         Deutsche Telekom Laboratories, TU Berlin           *
 *                         Ernst-Reuter-Platz 7, 10587 Berlin, Germany        *
 *                                                                            *
 * This file is part of the SoundScape Renderer (SSR).                        *
 *                                                                            *
 * The SSR is free software:  you can redistribute it and/or modify it  under *
 * the terms of the  GNU  General  Public  License  as published by the  Free *
 * Software Foundation, either version 3 of the License,  or (at your option) *
 * any later version.                                                         *
 *                                                                            *
 * The SSR is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY;  without even the implied warranty of MERCHANTABILITY or FITNESS *
 * FOR A PARTICULAR PURPOSE.                                                  *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * You should  have received a copy  of the GNU General Public License  along *
 * with this program.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * The SSR is a tool  for  real-time  spatial audio reproduction  providing a *
 * variety of rendering algorithms.                                           *
 *                                                                            *
 * http://tu-berlin.de/?id=ssr                  SoundScapeRenderer@telekom.de *
 ******************************************************************************/

/** @file
 * Command line parsing, and config-file related stuff (definition).
 *
 * $LastChangedDate: 2012-02-01 11:43:25 +0100 (Wed, 01 Feb 2012) $
 * $LastChangedRevision: 1659 $
 * $LastChangedBy: geier.matthias $
 **/

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#ifdef HAVE_CONFIG_H
#include <config.h> // for ENABLE_*, HAVE_*, WITH_*
#endif

#include <string>

#include "renderer.h"

#if !defined(ENABLE_NEWRENDERER)
#include "wfsrenderer.h"
#include "vbaprenderer.h"
#endif

namespace ssr
{
  /// program settings from config-file and command line
  struct conf_struct
  {
    std::string exec_name;                ///< name of executable (without path)
    std::string name;                     ///< name of renderer system
    float master_volume_correction;       ///< correction factor in dB
    float stand_ampl_ref_dist;            ///< distance where plane sources are
                                          ///< as loud as the other source types
    bool gui;                             ///< start GUI?
    bool ip_server;                       ///< start IP server?
    std::string tracker;                  ///< type of head tracker (or "")
    std::string tracker_ports;            ///< space-separated serial ports
    bool freewheeling;                    ///< use JACK's freewheeling mode?
    std::string scene_file_name;          ///< scene file to load
    std::string playback_setup_file_name; ///< reproduction setup to load
    std::string xml_schema;               ///< schema file to validate XML files
    std::string audio_recorder_file_name; ///< output file for audio recorder
    Renderer::renderer_t renderer;        ///< type of renderer to use
    std::string input_port_prefix;        ///< e.g. "alsa_pcm:capture"
    std::string output_port_prefix;       ///< e.g. "alsa_pcm:playback"
    std::string path_to_gui_images;       ///< dto.
    std::string path_to_scene_menu;       ///< path to scene_menu.conf

    int number_of_threads;
#ifdef ENABLE_IP_INTERFACE   
    int server_port;                      ///< listening port
#endif
    /// file name of WFS prefilter
    std::string wfs_prefilter_file_name;
#if defined(ENABLE_NEWRENDERER)
    /// size of delay line (in samples)
    unsigned int wfs_delayline_size;
    /// maximum negative delay (in samples, wfs_initial_delay >= 0)
    unsigned int wfs_initial_delay;
#else
    /// size of delay line (in samples)
    WFSRenderer::delay_size_t wfs_delayline_size;
    /// maximum negative delay (in samples, wfs_initial_delay >= 0)
    WFSRenderer::delay_size_t wfs_initial_delay;
#endif

    unsigned int hrir_size;
    std::string hrir_file_name;
    unsigned int ambisonics_order;
    bool         in_phase_rendering;

    bool loop; ///< temporary solution for looping sound files
  };

  bool parse_options(int& argc, char* argv[], conf_struct& conf);

  // static int parse(const char *line, char *key, char *value);
  // static int is_comment_or_empty(const char *line);
  int load_config_file(const char *filename, conf_struct& conf);
}

#endif

// Settings for Vim (http://www.vim.org/), please do not remove:
// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
